//
// UCSD p-System filesystem in user space
// Copyright (C) 2006 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or (at
// you option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston MA 02111-1307,
// USA
//

#include <lib/sector_io/smart_pointer.h>


sector_io_smart_pointer::~sector_io_smart_pointer()
{
    close();
}


sector_io_smart_pointer::sector_io_smart_pointer() :
    ref(0)
{
}


sector_io_smart_pointer::sector_io_smart_pointer(
	const sector_io_smart_pointer &arg) :
    ref(arg.ref)
{
    if (ref)
	ref->reference_count_up();
}


sector_io_smart_pointer &
sector_io_smart_pointer::operator=(const sector_io_smart_pointer &arg)
{
    if (this != &arg && ref != arg.ref)
    {
	sector_io *old = ref;
	ref = arg.ref;
	if (ref)
	    ref->reference_count_up();
	if (old)
	    old->reference_count_down();
    }
    return *this;
}


void
sector_io_smart_pointer::close()
{
    if (ref)
    {
	ref->reference_count_down();
	ref = 0;
    }
}
