#!/bin/sh
#
# UCSD p-System filesystem in user space
# Copyright (C) 2006 Peter Miller
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# you option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston MA 02111-1307,
# USA
#

TEST_SUBJECT="mknod"

for dir in . bl blbl blblbl blblblbl
do
    incl=$dir/aegis.supplementary/test_prelude.inc.sh
    if [ -f $incl ]
    then
       	. $incl
	break
    fi
done

#
# Create a disk image
#
$bindir/ucsdpsys_mkfs disk.image
test $? -eq 0 || fail

cp disk.image orig.image
test $? -eq 0 || no_result

mkdir mnt
test $? -eq 0 || no_result

#
# Mount the disk image.
#
$bindir/ucsdpsys_mount $testdir/disk.image $testdir/mnt
test $? -eq 0 || fail

#
# touch a file in that disk image
#
touch $testdir/mnt/abcdefgh
if test $? -ne 0
then
    fusermount -u $testdir/mnt
    fail
fi

#
# list the directory contents
#
cat > test.ok << 'fubar'
abcdefgh
fubar
if test $? -ne 0
then
    fusermount -u $testdir/mnt
    no_result
fi

ls $testdir/mnt > test.out
if test $? -ne 0
then
    fusermount -u $testdir/mnt
    fail
fi

#
# unmount the file system
#
fusermount -u $testdir/mnt

#
# Make sure the new file system.
#
$bindir/ucsdpsys_fsck disk.image
test $? -eq 0 || fail

#
# check the directory listing
#
diff test.ok test.out
test $? -eq 0 || fail

#
# Make sure that the file system image has been changed,
# not just the unix directory it was supposed to be mounted on.
#
cmp disk.image orig.image > /dev/null 2>&1 && fail

#
# The functionality exercised by this test worked.
# No other assertions are made.
#
pass
