#!/bin/sh
#
# UCSD p-System filesystem in user space
# Copyright (C) 2006 Peter Miller
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# you option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston MA 02111-1307,
# USA
#

TEST_SUBJECT="write"

for dir in . bl blbl blblbl blblblbl
do
    incl=$dir/aegis.supplementary/test_prelude.inc.sh
    if [ -f $incl ]
    then
       	. $incl
	break
    fi
done

#
# Create a disk image
#
$bindir/ucsdpsys_mkfs -i disk.image
test $? -eq 0 || fail

cp disk.image orig.image
test $? -eq 0 || no_result

mkdir mnt
test $? -eq 0 || no_result

#
# Mount the disk image.
#
$bindir/ucsdpsys_mount $testdir/disk.image $testdir/mnt
test $? -eq 0 || fail

umount_and_fail() {
    fusermount -u $testdir/mnt
    fail
}

umount_and_no_result() {
    fusermount -u $testdir/mnt
    no_result
}

#
# write a couple of files into the disk image
#
$bindir/test_rdwr -b $testdir/mnt/abcd
test $? -eq 0 || umount_and_fail

cat > efgh.ok << 'fubar'
This is the
test data.
fubar
test $? -eq 0 || umount_and_no_result

cp efgh.ok $testdir/mnt/efgh
test $? -eq 0 || umount_and_fail

#
# list the directory contents
#
cat > test.ok << 'fubar'
abcd
efgh
fubar
test $? -eq 0 || umount_and_no_result

ls $testdir/mnt > test.out
test $? -eq 0 || umount_and_fail

diff test.ok test.out
test $? -eq 0 || umount_and_fail

#
# Now write the first file again.  This will require the second file to
# be moved.
#
$bindir/test_rdwr -b $testdir/mnt/abcd
test $? -eq 0 || umount_and_fail

# Now check that the second file did not suffer when it was moved.
diff efgh.ok $testdir/mnt/efgh
test $? -eq 0 || umount_and_fail

#
# unmount the file system
#
fusermount -u $testdir/mnt

#
# Make sure the file system is OK.
#
$bindir/ucsdpsys_fsck disk.image
test $? -eq 0 || fail

#
# Make sure that the file system image has been changed, not just the
# unix directory it was supposed to be mounted on, and not just in
# memory.
#
cmp disk.image orig.image > /dev/null 2>&1 && fail

#
# The functionality exercised by this test worked.
# No other assertions are made.
#
pass
