#!/bin/sh
#
# UCSD p-System filesystem in user space
# Copyright (C) 2006 Peter Miller
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# you option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston MA 02111-1307,
# USA
#

TEST_SUBJECT="unlink"

for dir in . bl blbl blblbl blblblbl
do
    incl=$dir/aegis.supplementary/test_prelude.inc.sh
    if [ -f $incl ]
    then
       	. $incl
	break
    fi
done

#
# Create a disk image
#
$bindir/ucsdpsys_mkfs -B800 disk.image
test $? -eq 0 || fail

mkdir mnt
test $? -eq 0 || no_result

#
# Mount the disk image.
#
$bindir/ucsdpsys_mount $testdir/disk.image $testdir/mnt
test $? -eq 0 || fail

umount_and_fail() {
    fusermount -u $testdir/mnt
    fail
}

umount_and_no_result() {
    fusermount -u $testdir/mnt
    no_result
}

#
# write files into the disk image
#
date > $testdir/mnt/abcd
test $? -eq 0 || umount_and_no_result

date > $testdir/mnt/efgh
test $? -eq 0 || umount_and_no_result

#
# delete the first one
#
rm $testdir/mnt/abcd
test $? -eq 0 || umount_and_fail

#
# See how many are left
#
cat > test.ok << 'fubar'
efgh
fubar
test $? -eq 0 || umount_and_no_result

ls $testdir/mnt > test.out
test $? -eq 0 || umount_and_no_result

diff test.ok test.out
test $? -eq 0 || umount_and_fail

fusermount -u $testdir/mnt
test $? -eq 0 || umount_and_no_result

#
# Make sure the file system is OK.
#
$bindir/ucsdpsys_fsck disk.image
test $? -eq 0 || fail

#
# The functionality exercised by this test worked.
# No other assertions are made.
#
pass
