//
// UCSD p-System filesystem in user space
// Copyright (C) 2006 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or (at
// you option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston MA 02111-1307,
// USA
//

#include <cstdio>
#include <unistd.h>
#include <sys/vfs.h>

#include <lib/progname.h>
#include <lib/quit.h>
#include <lib/version.h>


static void
usage()
{
    const char *prog = progname_get();
    fprintf(stderr, "Usage: %s [ <option>... ] <filename>\n", prog);
    fprintf(stderr, "       %s -V\n", prog);
    quitter.exit(1);
}


int
main(int argc, char **argv)
{
    progname_set(argv[0]);
    for (;;)
    {
	int c = getopt(argc, argv, "V");
	if (c == EOF)
	    break;
	switch (c)
	{
	case 'V':
	    version_print();
	    return 0;

	default:
	    usage();
	}
    }
    if (optind + 1 != argc)
	usage();
    const char *filename = argv[optind];

    struct statfs st;
    if (statfs(filename, &st) < 0)
	quitter.fatal_error_with_errno("statfs %s", filename);

    printf("f_type = 0x%08lX\n", (long)st.f_type);
    printf("f_bsize = %9ld\n", (long)st.f_bsize);
    printf("f_blocks = %8ld\n", (long)st.f_blocks);
    printf("f_bfree = %9ld\n", (long)st.f_bfree);
    printf("f_bavail = %8ld\n", (long)st.f_bavail);
    printf("f_files = %9ld\n", (long)st.f_files);
    printf("f_ffree = %9ld\n", (long)st.f_ffree);
    printf("f_namelen = %7ld\n", (long)st.f_namelen);

    return 0;
}
