//
// UCSD p-System filesystem in user space
// Copyright (C) 2006 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or (at
// you option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston MA 02111-1307,
// USA
//

#ifndef UCSDPSYS_TEXT_FILTER_H
#define UCSDPSYS_TEXT_FILTER_H

#include <cstdio>

class rcstring; // forward

/**
  * The filter class is used to represent an abstratc filter to be
  * applied to files.
  */
class filter
{
public:
    /**
      * The destructor.
      */
    virtual ~filter();

    /**
      * The default constructor.
      */
    filter();

    /**
      * The run method is used to perform the actions of the filter on
      * one or more files.
      *
      * @param argc
      * @param argv
      */
    void run(int argc, char **argv);

protected:
    /**
      * The translate method is used to translate from the input to the output.
      *
      * @param ifp
      *     The input stream to be processed.
      * @param ifn
      *     The name of the input file, for error messages.
      * @param ofp
      *     The output stream onto which the translation output is to be
      *     written.
      * @param ofn
      *     The name of the output file, for error messages.
      */
    virtual void translate(FILE *ifp, const rcstring &ifn, FILE *ofp,
	const rcstring &ofn) = 0;

private:
    /**
      * The insitu method is used to translate a file in place.  A
      * temporary file is created in the source directory to hold the
      * translation output.  On success the temporary file is renamed
      * over the input file.  Note that this requires enough space on
      * the file system to hold both the old and new files, but only one
      * at a time.
      *
      * @param filename
      *     The name of the file to be translated.
      */
    void insitu(const rcstring &filename);

    /**
      * The copy constructor.  Do not use.
      */
    filter(const filter &);

    /**
      * The assignment operator.  Do not use.
      */
    filter &operator=(const filter &);
};

#endif // UCSDPSYS_TEXT_FILTER_H
