//
// UCSD p-System filesystem in user space
// Copyright (C) 2006 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or (at
// you option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston MA 02111-1307,
// USA
//

#ifndef UCSDPSYS_TEXT_FILTER_DECODE_H
#define UCSDPSYS_TEXT_FILTER_DECODE_H

#include <ucsdpsys_text/filter.h>

/**
  * The filter_decode class is used to represent a filter for
  * translating UCSD p-System text files into Unix text files.
  */
class filter_decode:
    public filter
{
public:
    /**
      * The destructor.
      */
    virtual ~filter_decode();

    /**
      * The default constructor.
      */
    filter_decode();

    // See base class for documentation.
    void translate(FILE *ifp, const rcstring &ifn, FILE *ofp,
	const rcstring &ofn);

private:
    /**
      * The column instance variable is used to remember the character
      * column number of the cursor on the current line.
      */
    int column;

    /**
      * The non_white instance variable is used to remember whether ot
      * no a non-white characters has yet been seen on the current line.
      */
    bool non_white;

    /**
      * The process method is used to process a block of text from UCSD
      * p-System format to Unix text format.
      *
      * This method does NOT assume maximally conforming text.
      *
      * @param buffer
      *     The data to be processed.
      * @param size
      *     The number of bytes of data to be processed.
      * @param ofp
      *     The output stream onto which the translation output is to be
      *     written.
      * @param ofn
      *     The name of the output file, for error messages.
      */
    void process(const unsigned char *buffer, size_t size, FILE *ofp,
	const rcstring &ofn);

    /**
      * The copy constructor.  Do not use.
      */
    filter_decode(const filter_decode &);

    /**
      * The assignment operator.  Do not use.
      */
    filter_decode &operator=(const filter_decode &);
};

#endif // UCSDPSYS_TEXT_FILTER_DECODE_H
