//
// UCSD p-System filesystem in user space
// Copyright (C) 2006 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or (at
// you option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston MA 02111-1307,
// USA
//

#include <cstdlib>
#include <unistd.h>

#include <lib/progname.h>
#include <lib/version.h>
#include <ucsdpsys_text/filter/decode.h>
#include <ucsdpsys_text/filter/encode.h>


static void
usage()
{
    const char *prog = progname_get();
    fprintf(stderr, "Usage: %s -d [ <filename>... ]\n", prog);
    fprintf(stderr, "       %s -e [ <filename>... ]\n", prog);
    fprintf(stderr, "       %s -V\n", prog);
    exit(1);
}


int
main(int argc, char **argv)
{
    progname_set(argv[0]);
    filter *action = 0;
    for (;;)
    {
	int c = getopt(argc, argv, "deV");
	if (c == EOF)
	    break;
	switch (c)
	{
	case 'd':
	    action = new filter_decode();
	    break;

	case 'e':
	    action = new filter_encode();
	    break;

	case 'V':
	    version_print();
	    return 0;

	default:
	    usage();
	}
    }
    if (!action)
	usage();
    action->run(argc - optind, argv + optind);
    return 0;
}
