/*
    This file is part of an implementation of the UCSD I.5 psuedo-machine.
    Copyright (C) 1994,1995,2006 Richard. W. Stearn

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Email: richard@rns-stearn.demon.co.uk
*/

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include "pme.h"
#include "unit_io.h"
#include "memory.h"
#include "console.h"

#define DEBUG_CONSOLE	0

extern int dflag;

extern struct unit_entry unit_table[];

extern mem_t memory[];

/* ------------------------------------------------------------------------- */
word unit_read_console( word unit_num, word buffer_addr, word length, word block )
{
char ch;

if ( DEBUG0 || DEBUG_CONSOLE )
	fprintf( stderr, "unit_read_console: %d, %d, %d, %d\n", unit_num, buffer_addr, length, block );

if ( unit_table[ unit_num ].fd < 0 )
	return EIONOVOL;

ch = getchar();
memory[ 0 ] = ch & 0xff;

return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_write_console( word unit_num, word buffer_addr, word length, word block )
{
word io_result;

if ( DEBUG0 || DEBUG_CONSOLE )
	fprintf( stderr, "unit_write_console: %d, %d, %d, %d\n", unit_num, buffer_addr, length, block );

if ( unit_table[ unit_num ].fd < 0 )
	return EIONOVOL;

io_result = write( unit_table[ unit_num ].fd, &(memory[ buffer_addr ]), length );
if ( io_result != length )
	fprintf( stderr, "write failed\n" );

return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_clear_console( word unit_num )
{
if ( DEBUG1 || DEBUG_CONSOLE )
	fprintf( stderr, "unit_clear_console: %d\n", unit_num );

if ( unit_table[ unit_num ].fd < 0 )
	return EIONOVOL;

return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_busy_console( word unit_num )
{
if ( DEBUG1 || DEBUG_CONSOLE )
	fprintf( stderr, "unit_busy_console: %d\n", unit_num );
return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_wait_console( word unit_num )
{
if ( DEBUG1 || DEBUG_CONSOLE )
	fprintf( stderr, "unit_wait_console: %d\n", unit_num );
return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_stat_console( word unit_num, word buffer_addr, word length, word dummy )
{
if ( DEBUG0 || DEBUG_CONSOLE )
	fprintf( stderr, "unit_stat_console: %d, %d, %d, %d\n", unit_num, buffer_addr, length, dummy );

if ( unit_table[ unit_num ].fd < 0 )
	return EIONOVOL;

return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_init_console( word unit_num )
{
if ( DEBUG1 || DEBUG_CONSOLE )
	fprintf( stderr, "unit_init_console: %d\n", unit_num );

if ( unit_table[ unit_num ].device != NULL )
	{
	unit_table[ unit_num ].fd = open( unit_table[ unit_num ].device, unit_table[ unit_num ].flags );
	if ( DEBUG1 || DEBUG_CONSOLE )
		{
		if ( unit_table[ unit_num ].fd < 0 )
			fprintf( stderr, "Failed to open" );
		else
			fprintf( stderr, "Opened" );
		fprintf( stderr, " %s\n", unit_table[ unit_num ].device );
		}
	}
if ( unit_table[ unit_num ].fd < 0 )
	return EIONOVOL;

return EIONOERROR;
}

/* ------------------------------------------------------------------------- */
word unit_halt_console( word unit_num )
{
if ( DEBUG1 || DEBUG_CONSOLE )
	fprintf( stderr, "unit_halt_console: %d\n", unit_num );
if ( ( unit_table[ unit_num ].device != NULL ) && ( unit_table[ unit_num ].fd >= 0 ) )
	{
	close( unit_table[ unit_num ].fd );
	if ( DEBUG1 || DEBUG_CONSOLE )
		fprintf( stderr, "Closed %s\n", unit_table[ unit_num ].device );
	}
return EIONOERROR;
}
