/*
    This file is part of an implementation of the UCSD I.5 psuedo-machine.
    Copyright (C) 1994,1995,2006 Richard. W. Stearn

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Email: richard@rns-stearn.demon.co.uk
*/

#include <stdio.h>
#include <ctype.h>
#include "pme.h"
#include "memory.h"

union data_type_map
	{
	byte b[ 4 ];
	word w[ 2 ] ;
	real r[ 1 ];
	};

mem_t memory[ MEM_SIZE ];
word sp;

/* -------------------------------------------------------------------------- */
void mem_wr_b( word addr, byte b )
{
memory[ addr ] = b & 0x0ff;
}

/* -------------------------------------------------------------------------- */
byte mem_rd_b( word addr )
{
byte b;

b = memory[ addr ] & 0x0ff;
return b & 0x0ff;
}

/* -------------------------------------------------------------------------- */
void mem_wr_w( word addr, word w )
{
union data_type_map map;

map.w[ 0 ] = w;
memory[ addr     ] = map.b[ 0 ];
memory[ addr + 1 ] = map.b[ 1 ];
}

/* -------------------------------------------------------------------------- */
word mem_rd_w( word addr )
{
union data_type_map map;

map.b[ 0 ] = memory[ addr     ];
map.b[ 1 ] = memory[ addr + 1 ];
return map.w[ 0 ];
}

/* -------------------------------------------------------------------------- */
void push_w( word w )
{
union data_type_map map;

map.w[ 0 ] = w;
sp--;
memory[ sp ] = map.b[ 1 ];
sp--;
memory[ sp ] = map.b[ 0 ];
}

/* -------------------------------------------------------------------------- */
word pop_w()
{
union data_type_map map;

map.b[ 0 ] = memory[ sp ];
sp++;
map.b[ 1 ] = memory[ sp ];
sp++;
return map.w[ 0 ];
}

/* -------------------------------------------------------------------------- */
void push_r( real r )
{
union data_type_map map;

map.r[ 0 ] = r;
push_w( map.w[ 1 ] );
push_w( map.w[ 0 ] );
}

/* -------------------------------------------------------------------------- */
real pop_r()
{
union data_type_map map;

map.w[ 0 ] = pop_w();
map.w[ 1 ] = pop_w();

return( map.r[ 0 ] );
}

/* -------------------------------------------------------------------------- */
void pop_s( set_t *set )
{
int i;

set->size = pop_w();
for ( i = 0; i < set->size; i++ )
	set->data[ i ] = pop_w();
}

/* -------------------------------------------------------------------------- */
void push_s( set_t *set )
{
int i;

for ( i = set->size; i > 0; i-- )
	push_w( set->data[ i - 1 ] );
push_w( set->size );
}

