/*
    This file is part of an implementation of the UCSD I.5 psuedo-machine.
    Copyright (C) 1994,1995,2006 Richard. W. Stearn

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Email: richard@rns-stearn.demon.co.uk
*/

#include <stdio.h>
#include "pme.h"
#include "ucsd_consts.h"
#include "ucsd_dir.h"
#include "memory.h"
#include "boot.h"
#include "unit_io.h"
#include "configfile.h"
#include "console.h"
#include "disk.h"
#include "graphic.h"
#include "remote.h"
#include "printer.h"
#include "null.h"

#define DEBUG_UNIT_IO	0

extern int dflag;

struct unit_entry unit_table[ MAX_UNITS ];

struct unit_io_operations unit_io_ops[ ] =
	{
		{
		unit_read_null,
		unit_write_null,
		unit_clear_null,
		unit_busy_null,
		unit_wait_null,
		unit_stat_null,
		unit_init_null,
		unit_halt_null
		},
		{
		unit_read_console,
		unit_write_console,
		unit_clear_console,
		unit_busy_console,
		unit_wait_console,
		unit_stat_console,
		unit_init_console,
		unit_halt_console
		},
		{
		unit_read_graphic,
		unit_write_graphic,
		unit_clear_graphic,
		unit_busy_graphic,
		unit_wait_graphic,
		unit_stat_graphic,
		unit_init_graphic,
		unit_halt_graphic
		},
		{
		unit_read_disk,
		unit_write_disk,
		unit_clear_disk,
		unit_busy_disk,
		unit_wait_disk,
		unit_stat_disk,
		unit_init_disk,
		unit_halt_disk
		},
		{
		unit_read_remote,
		unit_write_remote,
		unit_clear_remote,
		unit_busy_remote,
		unit_wait_remote,
		unit_stat_remote,
		unit_init_remote,
		unit_halt_remote
		},
		{
		unit_read_printer,
		unit_write_printer,
		unit_clear_printer,
		unit_busy_printer,
		unit_wait_printer,
		unit_stat_printer,
		unit_init_printer,
		unit_halt_printer
		},
	} ;

/* ------------------------------------------------------------------------- */
void init_unit_io( char *config_filename )
{
int vol;

for ( vol = 0; vol < MAX_UNITS; vol++ )
	{
	unit_table[ vol ].type = UNIT_NULL;
	unit_table[ vol ].device = NULL;
	unit_table[ vol ].fd = -1;
	unit_table[ vol ].flags = 0;
	}

read_config( config_filename );

for ( vol = 0; vol < MAX_UNITS; vol++ )
	if ( unit_init( vol ) )
		{
		if ( DEBUG5 || DEBUG_UNIT_IO )
			fprintf( stderr, "unit_init of unit %d failed\n", vol );
		}
}

/* ------------------------------------------------------------------------- */
void halt_unit_io()
{
int vol;

for ( vol = 0; vol < MAX_UNITS; vol++ )
	unit_halt( vol );
}

/* ------------------------------------------------------------------------- */
word unit_read( word unit_num, word buffer_addr, word length, word block, word flags )
{
word io_result;

if ( DEBUG1 || DEBUG_UNIT_IO )
	fprintf( stderr, "unit_read: %d, %d, %d, %d\n", unit_num, buffer_addr, length, block );
io_result = unit_io_ops[ unit_table[ unit_num ].type ].unit_read ( unit_num, buffer_addr, length, block );
if ( DEBUG1 || DEBUG_UNIT_IO )
	{
	fprintf( stderr, "unit_read: io_result = %d - " , io_result );
#ifdef _DEBUG_H_
	print_io_error( io_result );
#endif
	fprintf( stderr, "\n" );
	}
return io_result;
}

/* ------------------------------------------------------------------------- */
word unit_write( word unit_num, word buffer_addr, word length, word block, word flags )
{
word io_result;

if ( DEBUG1 || DEBUG_UNIT_IO )
	fprintf( stderr, "unit_write: %d, %d, %d, %d\n", unit_num, buffer_addr, length, block );
io_result = unit_io_ops[ unit_table[ unit_num ].type ].unit_write ( unit_num, buffer_addr, length, block );
if ( DEBUG1 || DEBUG_UNIT_IO )
	{
	fprintf( stderr, "unit_write: io_result = %d - " , io_result );
#ifdef _DEBUG_H_
	print_io_error( io_result );
#endif
	fprintf( stderr, "\n" );
	}
return io_result;
}

/* ------------------------------------------------------------------------- */
word unit_clear( word unit_num )
{
word io_result;

if ( DEBUG1 || DEBUG_UNIT_IO )
	fprintf( stderr, "unit_clear: %d\n", unit_num );
io_result = unit_io_ops[ unit_table[ unit_num ].type ].unit_clear ( unit_num );
if ( DEBUG1 || DEBUG_UNIT_IO )
	{
	fprintf( stderr, "unit_clear: io_result = %d - " , io_result );
#ifdef _DEBUG_H_
	print_io_error( io_result );
#endif
	fprintf( stderr, "\n" );
	}
return io_result;
}

/* ------------------------------------------------------------------------- */
word unit_busy( word unit_num )
{
word io_result;

if ( DEBUG1 || DEBUG_UNIT_IO )
	fprintf( stderr, "unit_busy: %d\n", unit_num );
io_result = unit_io_ops[ unit_table[ unit_num ].type ].unit_busy ( unit_num );
if ( DEBUG1 || DEBUG_UNIT_IO )
	{
	fprintf( stderr, "unit_busy: io_result = %d - " , io_result );
#ifdef _DEBUG_H_
	print_io_error( io_result );
#endif
	fprintf( stderr, "\n" );
	}
return io_result;
}

/* ------------------------------------------------------------------------- */
word unit_wait( word unit_num )
{
word io_result;

if ( DEBUG1 || DEBUG_UNIT_IO )
	fprintf( stderr, "unit_wait: %d\n", unit_num );
io_result = unit_io_ops[ unit_table[ unit_num ].type ].unit_wait ( unit_num );
if ( DEBUG1 || DEBUG_UNIT_IO )
	{
	fprintf( stderr, "unit_wait: io_result = %d - " , io_result );
#ifdef _DEBUG_H_
	print_io_error( io_result );
#endif
	fprintf( stderr, "\n" );
	}
return io_result;
}

/* ------------------------------------------------------------------------- */
word unit_stat( word unit_num, word buffer_addr, word length, word flags )
{
word io_result;

if ( DEBUG1 || DEBUG_UNIT_IO )
	fprintf( stderr, "unit_stat: %d, %d, %d, %d\n", unit_num, buffer_addr, length, flags );
io_result = unit_io_ops[ unit_table[ unit_num ].type ].unit_stat ( unit_num, buffer_addr, length, flags );
if ( DEBUG1 || DEBUG_UNIT_IO )
	{
	fprintf( stderr, "unit_stat: io_result = %d - " , io_result );
#ifdef _DEBUG_H_
	print_io_error( io_result );
#endif
	fprintf( stderr, "\n" );
	}
return io_result;
}

/* ------------------------------------------------------------------------- */
word unit_init( word unit_num )
{
word io_result;

if ( DEBUG1 || DEBUG_UNIT_IO )
	fprintf( stderr, "unit_init: %d\n", unit_num );
io_result = unit_io_ops[ unit_table[ unit_num ].type ].unit_init ( unit_num );
if ( DEBUG1 || DEBUG_UNIT_IO )
	{
	fprintf( stderr, "unit_init: io_result = %d - " , io_result );
#ifdef _DEBUG_H_
	print_io_error( io_result );
#endif
	fprintf( stderr, "\n" );
	}
return io_result;
}

/* ------------------------------------------------------------------------- */
word unit_halt( word unit_num )
{
word io_result;

if ( DEBUG1 || DEBUG_UNIT_IO )
	fprintf( stderr, "unit_halt: %d\n", unit_num );
io_result = unit_io_ops[ unit_table[ unit_num ].type ].unit_halt ( unit_num );
if ( DEBUG1 || DEBUG_UNIT_IO )
	{
	fprintf( stderr, "unit_halt: io_result = %d - " , io_result );
#ifdef _DEBUG_H_
	print_io_error( io_result );
#endif
	fprintf( stderr, "\n" );
	}
return io_result;
}
