/*
    This file is part of an implementation of the UCSD I.5 psuedo-machine.
    Copyright (C) 1994,1995,2006 Richard. W. Stearn

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Email: richard@rns-stearn.demon.co.uk
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <string.h>
#include <time.h>

#define byte	unsigned char
#define word	unsigned short

#include <ucsd_consts.h>
#include <ucsd_dir.h>

#define MIN( a, b )	( (a) < (b) ? (a) : (b) )

char *months[] =
	{
	"???",
	"Jan",
	"Feb",
	"Mar",
	"Apr",
	"May",
	"Jun",
	"Jul",
	"Aug",
	"Sep",
	"Oct",
	"Nov",
	"Dec",
	"???",
	"???",
	"???"
	} ;

char *ftypes[] =
	{
	"Untypedfile",
	"Xdskfile",
	"Codefile",
	"Textfile",
	"Infofile",
	"Datafile",
	"Graffile",
	"Fotofile",
	"Securedir",
	"Subsvol",
	NULL
	} ;

/* ------------------------------------------------------------------------- */
void put_file( int img_fd, char *filename, char *ftype )
{
word index;
word num_entries;
struct ucsd_dir_entry directory[ UCSD_MAXDIR ];
char tid[ UCSD_TIDLENG + 2 ];
int fd;
int read_size;
byte buffer[ UCSD_BLKSZ ];
int block;
struct tm *now_tm;
time_t now;
off_t offset;
int fkind;

offset = UCSD_DIRBLK * UCSD_BLKSZ;
if ( lseek( img_fd, offset, SEEK_SET ) != offset )
	{
	fprintf( stderr, "Failed to locate directory\n" );
	return;
	}
if ( read( img_fd, directory, sizeof( directory ) ) != sizeof( directory ) )
	{
	fprintf( stderr, "Failed to read directory\n" );
	return;
	}

sprintf( tid, "%-*.*s", UCSD_TIDLENG, UCSD_TIDLENG, filename );
for ( index = 0; index < UCSD_TIDLENG; index++ )
	tid[ index ] = toupper( tid[ index ] );

/* search the directory for the filename */
num_entries = MIN( directory[ 0 ].d.v_ent.dnumfiles, UCSD_MAXDIR - 1);
index = 1;
while ( ( index <= num_entries )
	&& ( strncasecmp( &(directory[ index ].d.t_ent.dtid[ 1 ]), tid, directory[ index ].d.t_ent.dtid[ 0 ] & 0xff ) != 0 )
	 )
		index++;

if ( index <= num_entries )
	{
	fprintf( stderr, "%s already exists\n", filename );
	return;
	}

if ( index >= UCSD_MAXDIR )
	{
	fprintf( stderr, "Directory is full\n" );
	return;
	}

/* should really look for the largest space, but index now points to the next
   available dir entry so we will use for the moment */
block = directory[ index - 1 ].dlastblk;
offset = block * UCSD_BLKSZ;
if ( lseek( img_fd, offset, SEEK_SET ) != offset )
	{
	fprintf( stderr, "Failed to locate start of new file\n" );
	return;
	}

fd = open( filename, O_RDONLY );
if ( fd < 0 )
	{
	fprintf( stderr, "Failed to open %s\n", filename );
	return;
	}

do
	{
	read_size = read( fd, buffer, UCSD_BLKSZ );
	if ( read_size < 0 )
		{
		fprintf( stderr, "Block read failed at block %d\n", block );
		return;
		}
	if ( write( img_fd, buffer, read_size ) != read_size )
		{
		fprintf( stderr, "Block write failed at block %d\n", block );
		return;
		}
	block++;
	} while ( read_size == UCSD_BLKSZ );

close( fd );

if ( read_size == 0 )
	{
	read_size = 512;
	block--;
	}

fkind = UNTYPEDFILE;
if ( ftype != NULL )
	{
	fkind = 0;
	while ( ( ftypes[ fkind ] != NULL )
		&& ( strncasecmp( ftype, ftypes[ fkind ], strlen( ftype ) ) != 0 )
		)
		fkind++;
	if ( ftypes[ fkind ] == NULL )
		fkind = UNTYPEDFILE;
	}

/* do the UCSD directory book keeping */
directory[ index ].dfirstblk = directory[ index - 1 ].dlastblk;
directory[ index ].dlastblk = block;
directory[ index ].dfkind = fkind;
memcpy( &(directory[ index ].d.t_ent.dtid[ 1 ]), tid, UCSD_TIDLENG );
directory[ index ].d.t_ent.dtid[ 0 ] = MIN( UCSD_TIDLENG, strlen( filename ) );
directory[ index ].d.t_ent.dlastbyte = read_size;

now = time( NULL );
now_tm = localtime( &now );
directory[ index ].d.t_ent.daccess.day = now_tm->tm_mday;
directory[ index ].d.t_ent.daccess.month = now_tm->tm_mon;
directory[ index ].d.t_ent.daccess.year = now_tm->tm_year % 100;

directory[ 0 ].d.v_ent.dnumfiles++;

lseek( img_fd, UCSD_DIRBLK * UCSD_BLKSZ, SEEK_SET );
if ( write( img_fd, directory, sizeof( directory ) ) != sizeof( directory ) )
	fprintf( stderr, "Failed to write UCSD directory\n" );
if ( directory[ 0 ].dlastblk >= 10 )
	{
	lseek( img_fd, (UCSD_DIRBLK + 4) * UCSD_BLKSZ, SEEK_SET );
	if ( write( img_fd, directory, sizeof( directory ) ) != sizeof( directory ) )
		fprintf( stderr, "Failed to write dup UCSD directory\n" );
	}
}

/* ------------------------------------------------------------------------- */
void get_file( int img_fd, char *filename )
{
word index;
word num_entries;
struct ucsd_dir_entry directory[ UCSD_MAXDIR ];
char tid[ UCSD_TIDLENG + 2 ];
int fd;
int read_size;
byte buffer[ UCSD_BLKSZ ];
int block;
off_t offset;

offset = UCSD_DIRBLK * UCSD_BLKSZ;
if ( lseek( img_fd, offset, SEEK_SET ) != offset )
	{
	fprintf( stderr, "Failed to locate directory\n" );
	return;
	}
if ( read( img_fd, directory, sizeof( directory ) ) != sizeof( directory ) )
	{
	fprintf( stderr, "Failed to read directory\n" );
	return;
	}

sprintf( tid, "%-*.*s", UCSD_TIDLENG, UCSD_TIDLENG, filename );
for ( index = 0; index < UCSD_TIDLENG; index++ )
	tid[ index ] = toupper( tid[ index ] );

/* search the directory for the filename */
num_entries = MIN( directory[ 0 ].d.v_ent.dnumfiles, UCSD_MAXDIR - 1 );
index = 1;
while ( ( index <= num_entries )
	&& ( strncasecmp( &(directory[ index ].d.t_ent.dtid[ 1 ]), tid, directory[ index ].d.t_ent.dtid[ 0 ] & 0xff ) != 0 )
	 )
		index++;

if ( index > num_entries )
	{
	fprintf( stderr, "Unable to find %s\n", filename );
	return;
	}

fd = open( filename, O_RDWR | O_CREAT, 0644 );
if ( fd < 0 )
	{
	fprintf( stderr, "Failed to open %s\n", filename );
	return;
	}

/* find the start of the file in the image */
block = directory[ index ].dfirstblk;
offset = block * UCSD_BLKSZ;
if ( lseek( img_fd, offset, SEEK_SET ) != offset )
	{
	fprintf( stderr, "Failed to locate start of file\n" );
	return;
	}

/* copy all the full blocks */
if ( ( directory[ index ].dlastblk - block ) > 1 )
	while ( block < (directory[ index ].dlastblk - 1) )
		{
		read_size = read( img_fd, buffer, UCSD_BLKSZ );
		if ( read_size != UCSD_BLKSZ )
			{
			fprintf( stderr, "Block read failed at block %d\n", block );
			return;
			}
		if ( write( fd, buffer, read_size ) != read_size )
			{
			fprintf( stderr, "Block write failed at block %d\n", block );
			return;
			}
		block++;
		}

/* now handle the possibly partial last or only block */
read_size = read( img_fd, buffer, directory[ index ].d.t_ent.dlastbyte );
if ( read_size != directory[ index ].d.t_ent.dlastbyte )
	{
	fprintf( stderr, "Block read failed at block %d\n", block );
	return;
	}
if ( write( fd, buffer, read_size ) != read_size )
	{
	fprintf( stderr, "Block write failed at block %d\n", block );
	return;
	}

close( fd );
}

/* ------------------------------------------------------------------------- */
void list_vid_entry( struct ucsd_dir_entry *entry )
{
fprintf( stdout, "%*.*s:%*c       %5u  %2u-%s-%02u %4u files, dir %u .. %u, lastboot %5u\n",
	entry->d.v_ent.dvid[ 0 ] & 0xff,
	entry->d.v_ent.dvid[ 0 ] & 0xff,
	&(entry->d.v_ent.dvid[ 1 ]),
	UCSD_VIDLENG - ( entry->d.v_ent.dvid[ 0 ] & 0xff ),
	' ',
	entry->d.v_ent.deovblk,
	entry->d.v_ent.dlastboot.day,
	months[ entry->d.v_ent.dlastboot.month ],
	entry->d.v_ent.dlastboot.year,
	entry->d.v_ent.dnumfiles,
	entry->dfirstblk,
	entry->dlastblk,
	entry->d.v_ent.dloadtime
	);
}

/* ------------------------------------------------------------------------- */
void list_tid_entry( struct ucsd_dir_entry *entry )
{
if ( ( entry->d.t_ent.dtid[ 0 ] & 0xff) <= UCSD_TIDLENG )
	fprintf( stdout, "%-*.*s %4u  %2u-%s-%02u %5u %5u %s\n",
		UCSD_TIDLENG,
		entry->d.t_ent.dtid[ 0 ] & 0xff,
		&(entry->d.t_ent.dtid[ 1 ]),
		entry->dlastblk - entry->dfirstblk,
		entry->d.t_ent.daccess.day,
		months[ entry->d.t_ent.daccess.month ],
		entry->d.t_ent.daccess.year,
		entry->dfirstblk,
		entry->d.t_ent.dlastbyte,
		ftypes[ entry->dfkind ]
		);
}

/* ------------------------------------------------------------------------- */
void dir_list( int img_fd, int vflag )
{
word index;
word num_entries;
struct ucsd_dir_entry directory[ UCSD_MAXDIR ];
off_t offset;

offset = UCSD_DIRBLK * UCSD_BLKSZ;
if ( lseek( img_fd, offset, SEEK_SET ) != offset )
	{
	fprintf( stderr, "Failed to locate directory\n" );
	return;
	}
if ( read( img_fd, directory, sizeof( directory ) ) != sizeof( directory ) )
	{
	fprintf( stderr, "Failed to read directory\n" );
	return;
	}

list_vid_entry( &(directory[ 0 ]) );
if ( ! vflag )
	{
	num_entries = MIN( directory[ 0 ].d.v_ent.dnumfiles, UCSD_MAXDIR - 1 );
	index = 1;
	while ( index <= num_entries )
			{
			list_tid_entry( &(directory[ index ]) );
			index++;
			}
	}
fprintf( stdout, "\n" );
}

/* ------------------------------------------------------------------------- */
void useage( char *progname )
{
fprintf( stderr, "Usage %s: [Hh?] [-t <type>] [-r|w <filename>] <image name>\n", progname );
fprintf( stderr, "\t-w <filename>    file to put\n" );
fprintf( stderr, "\t-r <filename>    file to get\n" );
fprintf( stderr, "\t-t <file type>   set the UCSD file type when writing to the image\n" );
fprintf( stderr, "\t-v               list volume id only\n" );
fprintf( stderr, "\t-H               help, you are reading it\n" );
fprintf( stderr, "\t-h               help, you are reading it\n" );
fprintf( stderr, "\t-?               help, you are reading it\n" );
fprintf( stderr, "\n" );
}

int main( int argc, char **argv )
{
int c;
int errflag = 0;
int hflag = 0;
int vflag = 0;
int rflag = 0;
int wflag = 0;
int img_fd = 0;
char *filename = NULL;
char *ftype = NULL;

char *progname;

extern char *optarg;
extern int optind;

/* First get the program name */
progname = (char *) strrchr( argv[ 0 ], '/' );
if ( progname == NULL )
	progname = argv[ 0 ];
else
	progname++;

while( ( c = getopt( argc, argv, "Hh?r:t:vw:" ) ) != EOF )
	switch ( c )
	{
	case( 'r' ):	filename = optarg;
			rflag++;
			break;
	case( 't' ):	ftype = optarg;
			break;
	case( 'w' ):	filename = optarg;
			wflag++;
			break;
	case( 'v' ):	vflag++;
			break;
	case( 'H' ):
	case( 'h' ):
	case( '?' ):	hflag++;
			break;
	default:
			break;
	}

if ( hflag || errflag || ( rflag && wflag ) )
	{
	useage( progname );
	return 1;
	}

if ( optind < argc ) /* There are additional arguments in the argument list */
	{
	if ( strlen( argv[ optind ] ) > 0 )
		{
		img_fd = open( argv[ optind ], O_RDWR );
		if ( img_fd < 0 )
			return 0;
		if ( filename != NULL )
			if ( rflag )
				get_file( img_fd, filename );
			else
				put_file( img_fd, filename, ftype );
		else
			dir_list( img_fd, vflag );
		close( img_fd );
		}
	}

return 0 ;
}
