
/*
    This file is part of an implementation of the UCSD I.5 psuedo-machine.
    Copyright (C) 1994,1995,2006 Richard. W. Stearn

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Email: richard@rns-stearn.demon.co.uk
*/

/* UCSD format textfile to unix (or DOS) file format */
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <string.h>

#define BLOCK_SIZE 1024

int convert_file( int fd, int dos )
{
int count;
int index;
int indent;
char buffer[ BLOCK_SIZE ];

if ( fd < 0 )
	return 1;

/* first block is editor info */
count = read( fd, buffer, BLOCK_SIZE );

/* the rest is text */
do
	{
	count = read( fd, buffer, BLOCK_SIZE );
	for ( index = 0; (index < count) && (buffer[ index ] != '\0'); index++ )
		switch ( buffer[ index ] )
			{
			case '\0': /* <nul> */
				/* throw away block end padding */
				break;
			case '\r': /* <cr> */
				if ( dos )
					fprintf( stdout,  "\r" );
				fprintf( stdout,  "\n" );
				break;
			case '\n': /* <lf> */
				break;
			case 16: /* <dle> */
				index++;
				indent = buffer[ index ] - ' ';
				if ( indent > 0 )
					fprintf( stdout,  "%*c", indent, ' ' );
				break;
			default:
				fprintf( stdout,  "%c", buffer[ index ] );
				break;
			}
	} while ( count > 0 );
return 0;
}

void useage( char *progname )
{
fprintf( stderr, "Usage %s: [DHh?]\n", progname );
fprintf( stderr, "          -D run as ucsd2dos\n" );
fprintf( stderr, "          -H help\n" );
fprintf( stderr, "          -h help\n" );
fprintf( stderr, "          -? help\n" );
fprintf( stderr, "\n" );
}

int main( int argc, char **argv )
{
int c;
int hflag = 0;
char *progname;

extern char *optarg;
extern int optind;

int dos = 0;

/* First get the program name */
progname = (char *) strrchr( argv[0], '/' );
if ( progname == NULL )
	progname = argv[0];
else
	progname++;

if ( strcasecmp( progname, "ucsd2dos" ) == 0 )
	dos++;

while((c = getopt(argc,argv,"DHh?")) != EOF)
	switch (c)
	{
	case('D'):	dos++;
			break;
	case('H'):
	case('h'):
	case('?'):	hflag++;
			break;
	default:
			break;
	}

if (hflag )
	{
	useage( progname );
	return 1;
	}

/* where are the additional arguments being presented ? */
if (optind < argc) /* There are additional arguments in the argument list */
	{
	while( optind < argc )
		{
		if ( strlen( argv[ optind ] ) > 0 )
			{
			int fd;

			fd = open( argv[optind], O_RDONLY );
			convert_file( fd, dos );
			close( fd );
			optind++;
			}
		}
	}
else /* The data is on stdin */
	convert_file( 0, dos );

return 0;
}
